@@ -1,4 +1,7 @@
 package cPanel::TaskQueue;
+BEGIN {
+  $cPanel::TaskQueue::VERSION = '0.500';
+}
 
 # cpanel - cPanel/TaskQueue.pm                    Copyright(c) 2010 cPanel, Inc.
 #                                                           All rights Reserved.
@@ -7,14 +10,14 @@ package cPanel::TaskQueue;
 # This module handles queuing of tasks for execution. The queue is persistent
 # handles consolidating of duplicate tasks.
 
+# ABSTRACT: Manage a FIFO queue of tasks to perform.
+
 use strict;
 #use warnings;
 use YAML::Syck             ();     # Data Serialization
 use cPanel::TaskQueue::Task();
 use cPanel::TaskQueue::Processor();
-use cPanel::CacheFile      ();
-
-our $VERSION = 0.400;
+use cPanel::StateFile      ();
 
 my $WNOHANG;
 if ( !exists $INC{'POSIX.pm'} ) {
@@ -49,7 +52,7 @@ sub import {
         my ($policy,$module) = splice( @_, 0, 2 );
         my @methods = ();
         if ( '-logger' eq $policy ) {
-            cPanel::CacheFile->import( '-logger' => $module );
+            cPanel::StateFile->import( '-logger' => $module );
         }
         else {
             die "Unrecognized policy '$policy'\n";
@@ -70,15 +73,15 @@ sub _first (&@) {  ## no critic(ProhibitSubroutinePrototypes)
 }
 
 # Namespace string used when creating task ids.
-my $taskqueue_uuid = "TaskQueue";
+my $taskqueue_uuid = 'TaskQueue';
 
 {
     # Class-wide definition of the valid processors
     my %valid_processors;
-    my $FILETYPE = "TaskQueue"; # Identifier at the beginning of the cache file
+    my $FILETYPE = 'TaskQueue'; # Identifier at the beginning of the state file
     my $CACHE_VERSION = 3; # Cache file version number.
 
-    # Disk Cache & cache file.
+    # State File
     #
     sub get_name                  { return $_[0]->{queue_name}; }
     sub get_default_timeout       { return $_[0]->{default_task_timeout}; }
@@ -86,6 +89,11 @@ my $taskqueue_uuid = "TaskQueue";
     sub get_max_running           { return $_[0]->{max_in_process}; }
     sub get_default_child_timeout { return $_[0]->{default_child_timeout}; }
 
+    # Processing pausing
+    sub pause_processing          { $_[0]->{paused} = 1; return; }
+    sub resume_processing         { $_[0]->{paused} = 0; return; }
+    sub is_paused                 { return $_[0]->{paused} || 0; }
+
     # --------------------------------------
     # Class methods
 
@@ -93,13 +101,13 @@ my $taskqueue_uuid = "TaskQueue";
         my ($class, $command, $processor) = @_;
 
         unless ( defined $command and length $command ) {
-            cPanel::CacheFile->_throw( "Missing command in register_task_processor.\n" );
+            cPanel::StateFile->_throw( "Missing command in register_task_processor.\n" );
         }
         unless ( defined $processor ) {
-            cPanel::CacheFile->_throw( "Missing task processor in register_task_processor.\n" );
+            cPanel::StateFile->_throw( "Missing task processor in register_task_processor.\n" );
         }
         if ( exists $valid_processors{$command} ) {
-            cPanel::CacheFile->_throw( "Command '$command' already has a TaskQueue::Processor registered.\n" );
+            cPanel::StateFile->_throw( "Command '$command' already has a TaskQueue::Processor registered.\n" );
         }
         if( 'CODE' eq ref $processor ) {
             $valid_processors{$command} = cPanel::TaskQueue::Processor::CodeRef->new( {code =>$processor} );
@@ -109,17 +117,17 @@ my $taskqueue_uuid = "TaskQueue";
             return 1;
         }
 
-        cPanel::CacheFile->_throw( "Unrecognized task processor object.\n" );
+        cPanel::StateFile->_throw( "Unrecognized task processor object.\n" );
     }
 
     sub unregister_task_processor {
         my ($class, $command) = @_;
 
         unless ( defined $command and length $command ) {
-            cPanel::CacheFile->_throw( "Missing command in unregister_task_processor.\n" );
+            cPanel::StateFile->_throw( "Missing command in unregister_task_processor.\n" );
         }
         unless ( exists $valid_processors{$command} ) {
-            cPanel::CacheFile->_throw( "Command '$command' not registered, ignoring.\n" );
+            cPanel::StateFile->_throw( "Command '$command' not registered, ignoring.\n" );
             return;
         }
 
@@ -130,57 +138,62 @@ my $taskqueue_uuid = "TaskQueue";
     # Initialize parameters.
     sub new {
         my ( $class, $args_ref ) = @_;
-        cPanel::CacheFile->_throw( "Args parameter must be a hash reference\n" ) unless 'HASH' eq ref $args_ref;
+        cPanel::StateFile->_throw( "Args parameter must be a hash reference\n" ) unless 'HASH' eq ref $args_ref;
 
-        cPanel::CacheFile->_throw( "No caching directory supplied.\n" ) unless exists $args_ref->{cache_dir};
-        cPanel::CacheFile->_throw( "No queue name supplied.\n" ) unless exists $args_ref->{name};
+        # Deprecate the state_dir argument, replace with state_dir
+        $args_ref->{state_dir} ||= $args_ref->{cache_dir} if exists $args_ref->{cache_dir};
+        cPanel::StateFile->_throw( "No state directory supplied.\n" ) unless exists $args_ref->{state_dir};
+        cPanel::StateFile->_throw( "No queue name supplied.\n" ) unless exists $args_ref->{name};
 
         # TODO: Do I want to sanity check the arguments?
         my $self = bless {
-            queue_name => $args_ref->{name},
-            default_task_timeout => 60,
-            max_task_timeout => 300,
-            max_in_process => 2,
+            queue_name            => $args_ref->{name},
+            default_task_timeout  => 60,
+            max_task_timeout      => 300,
+            max_in_process        => 2,
             default_child_timeout => 3600,
-            disk_cache_file => "$args_ref->{cache_dir}/$args_ref->{name}_queue.yaml",
-            next_id => 1,
-            queue_waiting => [],
-            processing_list => [],
-            disk_cache => undef,
+            disk_state_file       => "$args_ref->{state_dir}/$args_ref->{name}_queue.yaml",
+            next_id               => 1,
+            queue_waiting         => [],
+            processing_list       => [],
+            deferral_queue        => [],
+            disk_state            => undef,
+            defer_obj             => undef,
+            paused                => 0,
         }, $class;
 
         # Make a disk file to track the object.
-        my $cache_args = {
-            cache_file=>$self->{disk_cache_file}, data_obj => $self,
-            exists $args_ref->{cache_timeout} ? (timeout => $args_ref->{cache_timeout}) : (),
+        my $state_args = {
+            state_file=>$self->{disk_state_file}, data_obj => $self,
+            exists $args_ref->{state_timeout} ? (timeout => $args_ref->{state_timeout}) : (),
             exists $args_ref->{logger} ? (logger => $args_ref->{logger}) : (),
         };
         eval {
-            $self->{disk_cache} = cPanel::CacheFile->new( $cache_args );
-        };
-        if ( $@ ) {
-            my $ex = $@;
+            $self->{disk_state} = cPanel::StateFile->new( $state_args );
+            1;
+        } or do {
+            my $ex = $@ || 'Unreocognized exception.';
             # If not a loading error, rethrow.
-            cPanel::CacheFile->_throw( $ex ) unless $ex =~ /Not a recognized|Invalid version/;
-            cPanel::CacheFile->_warn( $ex );
-            cPanel::CacheFile->_warn( "Moving bad cache file and retry.\n" );
-            cPanel::CacheFile->_notify(
+            cPanel::StateFile->_throw( $ex ) unless $ex =~ /Not a recognized|Invalid version/;
+            cPanel::StateFile->_warn( $ex );
+            cPanel::StateFile->_warn( "Moving bad state file and retry.\n" );
+            cPanel::StateFile->_notify(
                 'Unable to load TaskQueue metadata',
-                "Loading of [$self->{disk_cache_file}] failed: $ex\n"
-                . "Moving bad file to [$self->{disk_cache_file}.broken] and retrying.\n"
+                "Loading of [$self->{disk_state_file}] failed: $ex\n"
+                . "Moving bad file to [$self->{disk_state_file}.broken] and retrying.\n"
             );
-            unlink "$self->{disk_cache_file}.broken";
-            rename $self->{disk_cache_file}, "$self->{disk_cache_file}.broken"; 
+            unlink "$self->{disk_state_file}.broken";
+            rename $self->{disk_state_file}, "$self->{disk_state_file}.broken"; 
 
-            $self->{disk_cache} = cPanel::CacheFile->new( $cache_args );
-        }
+            $self->{disk_state} = cPanel::StateFile->new( $state_args );
+        };
 
         # Use incoming parameters to override what's in the file.
-        if ( grep { exists $args_ref->{$_ } } qw/default_timeout max_timeout max_running default_child_timeout/ ) {
-            my $guard = $self->{disk_cache}->synch();
-            $self->{default_task_timeout} = $args_ref->{default_timeout}        if exists $args_ref->{default_timeout};
-            $self->{max_task_timeout} = $args_ref->{max_timeout}                if exists $args_ref->{max_timeout};
-            $self->{max_in_process} = $args_ref->{max_running}                  if exists $args_ref->{max_running};
+        if ( grep { exists $args_ref->{$_} } qw/default_timeout max_timeout max_running default_child_timeout/ ) {
+            my $guard = $self->{disk_state}->synch();
+            $self->{default_task_timeout}  = $args_ref->{default_timeout}       if exists $args_ref->{default_timeout};
+            $self->{max_task_timeout}      = $args_ref->{max_timeout}           if exists $args_ref->{max_timeout};
+            $self->{max_in_process}        = $args_ref->{max_running}           if exists $args_ref->{max_running};
             $self->{default_child_timeout} = $args_ref->{default_child_timeout} if exists $args_ref->{default_child_timeout};
             $guard->update_file();
         }
@@ -190,15 +203,15 @@ my $taskqueue_uuid = "TaskQueue";
 
     sub throw {
         my $self = shift;
-        return $self->{disk_cache} ? $self->{disk_cache}->throw( @_ ) : cPanel::CacheFile->_throw( @_ );
+        return $self->{disk_state} ? $self->{disk_state}->throw( @_ ) : cPanel::StateFile->_throw( @_ );
     }
     sub warn {
         my $self = shift;
-        return $self->{disk_cache} ? $self->{disk_cache}->warn( @_ ) : warn @_;
+        return $self->{disk_state} ? $self->{disk_state}->warn( @_ ) : warn @_;
     }
     sub info {
         my $self = shift;
-        return $self->{disk_cache} ? $self->{disk_cache}->info( @_ ) : undef;
+        return $self->{disk_state} ? $self->{disk_state}->info( @_ ) : undef;
     }
     # -------------------------------------------------------
     # Public methods
@@ -208,44 +221,49 @@ my $taskqueue_uuid = "TaskQueue";
         local $/;
         my ($magic, $version, $meta) = YAML::Syck::Load( scalar <$fh> );
 
-        $self->throw( "Not a recognized TaskQueue cache." ) unless $magic eq $FILETYPE;
-        $self->throw( "Invalid version of TaskQueue cache." ) unless $version eq $CACHE_VERSION;
+        $self->throw( 'Not a recognized TaskQueue state file.' ) unless $magic eq $FILETYPE;
+        $self->throw( 'Invalid version of TaskQueue state file.' ) unless $version eq $CACHE_VERSION;
 
         # Next id should continue increasing.
         #   (We might want to deal with wrap-around at some point.)
         $self->{next_id} = $meta->{nextid} if $meta->{nextid} > $self->{next_id};
 
         # TODO: Add more sanity checks here.
-        $self->{default_task_timeout} = $meta->{def_task_to}   if $meta->{def_task_to} > 0;
-        $self->{max_task_timeout} = $meta->{max_task_to}       if $meta->{max_task_to} > 0;
-        $self->{max_in_process} = $meta->{max_running}         if $meta->{max_running} > 0;
+        $self->{default_task_timeout}  = $meta->{def_task_to}  if $meta->{def_task_to} > 0;
+        $self->{max_task_timeout}      = $meta->{max_task_to}  if $meta->{max_task_to} > 0;
+        $self->{max_in_process}        = $meta->{max_running}  if $meta->{max_running} > 0;
         $self->{default_child_timeout} = $meta->{def_child_to} if $meta->{def_child_to} > 0;
+        $self->{paused}                = 1                     if exists $meta->{paused} && $meta->{paused};
+        $self->{defer_obj}             = exists $meta->{defer_obj} ? $meta->{defer_obj} : undef;
 
         # Clean queues that have been read from disk.
-        $self->{queue_waiting} = [
-            grep { defined $_ and eval { $_->isa( 'cPanel::TaskQueue::Task' ) } }
-            @{$meta->{waiting_queue}}
-        ];
-        $self->{processing_list} = [
-            grep { defined $_ and eval { $_->isa( 'cPanel::TaskQueue::Task' ) } }
-            @{$meta->{processing_queue}}
-        ];
+        $self->{queue_waiting}   = _clean_task_list( $meta->{waiting_queue} );
+        $self->{processing_list} = _clean_task_list( $meta->{processing_queue} );
+        $self->{deferral_queue}  = _clean_task_list( $meta->{deferral_queue} );
 
         return 1;
     }
 
+    sub _clean_task_list {
+        my ($task_list) = @_;
+        return [] unless defined $task_list;
+        return [ grep { defined $_ and eval { $_->isa( 'cPanel::TaskQueue::Task' ) } } @{ $task_list } ];
+    }
 
     sub save_to_cache {
         my ($self,$fh) = @_;
 
         my $meta = {
-            nextid => $self->{next_id},
-            def_task_to => $self->{default_task_timeout},
-            max_task_to => $self->{max_task_timeout},
-            max_running => $self->{max_in_process},
-            def_child_to => $self->{default_child_timeout},
-            waiting_queue => $self->{queue_waiting},
+            nextid           => $self->{next_id},
+            def_task_to      => $self->{default_task_timeout},
+            max_task_to      => $self->{max_task_timeout},
+            max_running      => $self->{max_in_process},
+            def_child_to     => $self->{default_child_timeout},
+            waiting_queue    => $self->{queue_waiting},
             processing_queue => $self->{processing_list},
+            deferral_queue   => $self->{deferral_queue},
+            paused           => ( $self->{paused} ? 1 : 0 ),
+            defer_obj        => $self->{defer_obj},
         };
         return print $fh YAML::Syck::Dump( $FILETYPE, $CACHE_VERSION, $meta );
     }
@@ -253,7 +271,7 @@ my $taskqueue_uuid = "TaskQueue";
     sub queue_task {
         my ( $self, $command ) = @_;
 
-        $self->throw( "Cannot queue an empty command." ) unless defined $command;
+        $self->throw( 'Cannot queue an empty command.' ) unless defined $command;
 
         if ( eval { $command->isa( 'cPanel::TaskQueue::Task' ) } ) {
             if ( 0 == $command->retries_remaining() ) {
@@ -265,11 +283,15 @@ my $taskqueue_uuid = "TaskQueue";
         }
 
         # must have non-space characters to be a command.
-        $self->throw( "Cannot queue an empty command." ) unless $command =~ /\S/;
+        $self->throw( 'Cannot queue an empty command.' ) unless $command =~ /\S/;
 
         my $task = cPanel::TaskQueue::Task->new(
-            {cmd=>$command, nsid=>$taskqueue_uuid, id=>$self->{next_id}++,
-             timeout=>$self->{default_child_timeout}}
+            {
+                cmd     => $command,
+                nsid    => $taskqueue_uuid,
+                id      => $self->{next_id}++,
+                timeout => $self->{default_child_timeout},
+            }
         );
         return $self->_queue_the_task( $task );
     }
@@ -278,11 +300,11 @@ my $taskqueue_uuid = "TaskQueue";
         my ( $self, $uuid ) = @_;
 
         unless ( _is_valid_uuid( $uuid ) ) {
-            $self->throw( "No Task uuid argument passed to unqueue_cmd." );
+            $self->throw( 'No Task uuid argument passed to unqueue_cmd.' );
         }
 
         # Lock the queue before we begin accessing it.
-        my $guard = $self->{disk_cache}->synch();
+        my $guard = $self->{disk_state}->synch();
         my $old_count = @{$self->{queue_waiting}};
 
         $self->{queue_waiting} = [ grep { $_->uuid() ne $uuid } @{$self->{queue_waiting}} ];
@@ -291,36 +313,33 @@ my $taskqueue_uuid = "TaskQueue";
         return $old_count > @{$self->{queue_waiting}};
     }
 
-    sub is_task_queued {
-        my ( $self, $uuid ) = @_;
+    sub _is_task_in_list {
+        my ( $self, $uuid, $list, $subname ) = @_;
 
         unless ( _is_valid_uuid( $uuid ) ) {
-            $self->throw( "No Task uuid argument passed to is_task_queued." );
+            $self->throw( "No Task uuid argument passed to $subname." );
         }
 
         # Update from disk, but don't worry about lock. Information only.
-        $self->{disk_cache}->synch();
-        return defined _first { $_->uuid() eq $uuid } @{$self->{queue_waiting}};
+        $self->{disk_state}->synch();
+        return defined _first { $_->uuid() eq $uuid } @{$self->{$list}};
     }
 
-    sub is_task_processing {
-        my ( $self, $uuid ) = @_;
+    sub is_task_queued     { return $_[0]->_is_task_in_list( $_[1], 'queue_waiting',   'is_task_queued' ); }
+    sub is_task_processing { return $_[0]->_is_task_in_list( $_[1], 'processing_list', 'is_task_processing' ); }
+    sub is_task_deferred   { return $_[0]->_is_task_in_list( $_[1], 'deferral_queue',  'is_task_deferred' ); }
 
-        unless ( _is_valid_uuid( $uuid ) ) {
-            $self->throw( "No Task uuid argument passed to is_task_processing" );
-        }
-
-        # Update from disk, but don't worry about lock. Information only.
-        $self->{disk_cache}->synch();
-        return defined _first { $_->uuid() eq $uuid } @{$self->{processing_list}};
+    sub _list_of_all_tasks {
+        my ($self) = @_;
+        return @{$self->{queue_waiting}}, @{$self->{deferral_queue}}, @{$self->{processing_list}};
     }
 
     sub find_task {
         my ($self, $uuid) = @_;
 
         # Update from disk, but don't worry about lock. Information only.
-        $self->{disk_cache}->synch();
-        my $task = _first { $_->uuid() eq $uuid } @{$self->{queue_waiting}}, @{$self->{processing_list}};
+        $self->{disk_state}->synch();
+        my $task = _first { $_->uuid() eq $uuid } $self->_list_of_all_tasks();
 
         return unless defined $task;
         return $task->clone();
@@ -330,8 +349,8 @@ my $taskqueue_uuid = "TaskQueue";
         my ($self, $command) = @_;
 
         # Update from disk, but don't worry about lock. Information only.
-        $self->{disk_cache}->synch();
-        my $task = _first { $_->command() eq $command } @{$self->{queue_waiting}}, @{$self->{processing_list}};
+        $self->{disk_state}->synch();
+        my $task = _first { $_->command() eq $command } $self->_list_of_all_tasks();
 
         return unless defined $task;
         return $task->clone();
@@ -341,36 +360,36 @@ my $taskqueue_uuid = "TaskQueue";
         my ($self, $command) = @_;
 
         # Update from disk, but don't worry about lock. Information only.
-        $self->{disk_cache}->synch();
-        my @tasks = grep { $_->command() eq $command } @{$self->{processing_list}}, @{$self->{queue_waiting}};
+        $self->{disk_state}->synch();
+        my @tasks = grep { $_->command() eq $command } $self->_list_of_all_tasks();
 
         return unless @tasks;
         return map { $_->clone() } @tasks;
     }
 
-    sub how_many_queued {
-        my ($self) = @_;
+    sub _how_many {
+        my ($self, $listname) = @_;
 
         # Update from disk, but don't worry about lock. Information only.
-        $self->{disk_cache}->synch();
-        return scalar @{$self->{queue_waiting}};
+        $self->{disk_state}->synch();
+        return scalar @{$self->{$listname}};
     }
 
-    sub how_many_in_process {
-        my ($self) = @_;
 
-        # Update from disk, but don't worry about lock. Information only.
-        $self->{disk_cache}->synch();
-        return scalar @{$self->{processing_list}};
-    }
+    sub how_many_queued     { return $_[0]->_how_many( 'queue_waiting' ); }
+    sub how_many_deferred   { return $_[0]->_how_many( 'deferral_queue' ); }
+    sub how_many_in_process { return $_[0]->_how_many( 'processing_list' ); }
 
     sub has_work_to_do {
         my ($self) = @_;
 
         # Update from disk, but don't worry about lock. Possibly information only.
-        $self->{disk_cache}->synch();
+        $self->{disk_state}->synch();
         $self->_clean_completed_tasks();
 
+        # If we are paused, there is no work to do.
+        return if $self->{paused};
+
         return scalar(@{$self->{processing_list}}) < $self->{max_in_process} && 0 != @{$self->{queue_waiting}};
     }
 
@@ -378,7 +397,7 @@ my $taskqueue_uuid = "TaskQueue";
         my ($self) = @_;
 
         # Update from disk, but don't worry about lock. Information only.
-        $self->{disk_cache}->synch();
+        $self->{disk_state}->synch();
         return unless @{$self->{queue_waiting}};
 
         return $self->{queue_waiting}->[0]->clone();
@@ -388,33 +407,46 @@ my $taskqueue_uuid = "TaskQueue";
         my ($self) = @_;
 
         # Lock the queue before doing any manipulations.
-        my $guard = $self->{disk_cache}->synch();
+        my $guard = $self->{disk_state}->synch();
 
         $self->_handle_already_running_tasks( $guard );
 
         if ( _first { !defined $_ } @{$self->{queue_waiting}} ) {
             # Somehow some undefined tasks got into the queue, log and
             # delete them.
-            $self->warn( "Undefined tasks found in the queue, removing..." );
+            $self->warn( 'Undefined tasks found in the queue, removing...' );
             $self->{queue_waiting} = [ grep { defined $_ } @{$self->{queue_waiting}} ];
             # Since we've changed the wait queue, we need to update disk file,
             # otherwise changes could be lost if we return early, below.
             $guard->update_file();
         }
-        # We can now schedule new tasks
-        return 1 unless @{$self->{queue_waiting}};
-        my $task = shift @{$self->{queue_waiting}};
 
-        # can fail if the processor for this command was removed.
-        unless ( exists $valid_processors{$task->command()} ) {
-            # TODO: log missing processor.
-            $self->warn( "No processor found for '".$task->command()."'." );
-            return 1;
+        # If we are paused, there is no work to do.
+        return 1 if $self->{paused};
+
+        my ($task, $processor);
+        while( !$task )  {
+            # We can now schedule new tasks
+            return 1 unless @{$self->{queue_waiting}};
+            $task = shift @{$self->{queue_waiting}};
+
+            # can fail if the processor for this command was removed.
+            $processor = _get_task_processor( $task );
+            unless ( $processor ) {
+                # TODO: log missing processor.
+                $self->warn( q{No processor found for '} . $task->command() . q{'.} );
+                return 1;
+            }
+            # Check for deferrals.
+            if( $processor->is_task_deferred( $task, $self->{defer_obj} ) ) {
+                unshift @{ $self->{deferral_queue} }, $task;
+                $task = undef;
+            }
         }
-        my $processor = $valid_processors{$task->command()};
 
         $task->begin();
         push @{$self->{processing_list}}, $task;
+        $self->_add_task_to_deferral_object( $task, $processor );
 
         # Finished making changes, save to disk.
         $guard->update_file();
@@ -430,9 +462,11 @@ my $taskqueue_uuid = "TaskQueue";
                     $orig_alarm = alarm( $self->_timeout( $processor ) );
                     $pid = $processor->process_task( $task->clone() );
                     alarm $orig_alarm;
+                    1;
+                } or do {
+                    $ex = $@; # save exception for later
+                    alarm $orig_alarm;
                 };
-                $ex = $@; # save exception for later
-                alarm $orig_alarm if $@;
             }
         );
 
@@ -444,13 +478,14 @@ my $taskqueue_uuid = "TaskQueue";
             my $uuid = $task->uuid();
             # remove finished item from the list.
             $self->{processing_list} = [ grep { $_->uuid() ne $uuid } @{$self->{processing_list}} ];
+            $self->_remove_task_from_deferral_object();
         }
 
         # Don't lose any exceptions.
         if ( $ex ) {
             if ( $ex eq "time out reached\n" ) {
                 # TODO: log timeout condition.
-                $self->warn( "Task '" . $task->full_command() . "' timed out during processing." );
+                $self->warn( q{Task '} . $task->full_command() . q{' timed out during processing.} );
             }
             else {
                 $self->throw( $ex );
@@ -466,7 +501,7 @@ my $taskqueue_uuid = "TaskQueue";
         my ($self) = @_;
 
         # Lock the queue for manipulation and to reduce new task items.
-        my $guard = $self->{disk_cache}->synch();
+        my $guard = $self->{disk_state}->synch();
         while ( @{$self->{processing_list}} ) {
             # we still need to remove some
             my $pid;
@@ -479,6 +514,7 @@ my $taskqueue_uuid = "TaskQueue";
                 grep { 0 == waitpid( $_->pid(), $WNOHANG ) }
                 grep { $_->pid() && $_->pid() != $pid } @{$self->{processing_list}}
             ];
+            $self->_process_deferrals();
             $guard->update_file();
         }
         return;
@@ -488,28 +524,33 @@ my $taskqueue_uuid = "TaskQueue";
         my ($self) = @_;
 
         # Update from disk, but don't worry about lock. Information only.
-        $self->{disk_cache}->synch();
+        $self->{disk_state}->synch();
 
         return {
-            waiting => [ map { $_->clone() } @{$self->{queue_waiting}} ],
-            processing => [ map { $_->clone() } @{$self->{processing_list}} ],
+            waiting    => [ map { $_->clone() } @{ $self->{queue_waiting} } ],
+            processing => [ map { $_->clone() } @{ $self->{processing_list} } ],
+            deferred   => [ map { $_->clone() } @{ $self->{deferral_queue} } ],
         };
     }
 
     # ---------------------------------------------------------------
     #  Private Methods.
 
+    sub _get_task_processor {
+        my ($task) = @_;
+        return $valid_processors{ $task->command() };
+    }
     # Test whether the supplied task descriptor duplicates any in the queue.
     sub _is_duplicate_command {
         my ($self, $task) = @_;
-        my $proc = $valid_processors{$task->command()};
+        my $proc = _get_task_processor( $task );
 
         return defined _first { $proc->is_dupe( $task, $_ ) } reverse @{$self->{queue_waiting}}; 
     }
 
     sub _process_overrides {
         my ($self, $task) = @_;
-        my $proc = $valid_processors{$task->command()};
+        my $proc = _get_task_processor( $task );
 
         $self->{queue_waiting} = [
             grep { !$proc->overrides( $task, $_ ) } @{$self->{queue_waiting}}
@@ -526,16 +567,17 @@ my $taskqueue_uuid = "TaskQueue";
 
         # Validate the incoming task.
         # It must be a command we recognize, have valid parameters, and not be a duplicate.
-        unless ( exists $valid_processors{$task->command()} ) {
-            $self->throw( "No known processor for '" . $task->command() ."'." );
+        my $proc = _get_task_processor( $task );
+        unless ( $proc ) {
+            $self->throw( q{No known processor for '} . $task->command() . q{'.} );
         }
-        unless ( $valid_processors{$task->command()}->is_valid_args( $task ) ) {
-            $self->throw( "Requested command [". $task->full_command() ."] has invalid arguments." );
+        unless ( $proc->is_valid_args( $task ) ) {
+            $self->throw( q{Requested command [} . $task->full_command() . q{] has invalid arguments.} );
         }
 
         # Lock the queue here, because we begin looking what's currently in the queue
         #  and don't want it to change under us.
-        my $guard = $self->{disk_cache}->synch();
+        my $guard = $self->{disk_state}->synch();
         # Check overrides first and then duplicate. This seems backward, but
         # actually is not. See the tests labelled 'override, not dupe' in
         # t/07.task_queue_dupes_and_overrides.t for the case that makes sense.
@@ -579,7 +621,7 @@ my $taskqueue_uuid = "TaskQueue";
 
         # Was not locked, so we need to lock and remove completed tasks again.
         if ( !$guard ) {
-            $guard = $self->{disk_cache}->synch();
+            $guard = $self->{disk_state}->synch();
             $self->_remove_completed_tasks_from_list();
         }
         $guard->update_file();
@@ -594,9 +636,60 @@ my $taskqueue_uuid = "TaskQueue";
             grep { $_->pid() && 0 == waitpid( $_->pid(), $WNOHANG ) }
             @{$self->{processing_list}}
         ];
+        $self->_process_deferrals();
         return;
     }
 
+    sub _add_task_to_deferral_object {
+        my ($self, $task, $processor) = @_;
+        return unless $task;
+
+        $processor ||= _get_task_processor( $task );
+        $self->{defer_obj}->{$_} = 1 foreach $processor->deferral_tags( $task );
+        return;
+    }
+
+    sub _remove_task_from_deferral_object {
+        my ($self, $task, $processor) = @_;
+        return unless $task;
+
+        $processor ||= _get_task_processor( $task );
+        delete $self->{defer_obj}->{$_} foreach $processor->deferral_tags( $task );
+        return;
+    }
+
+    # Clean up the object that tracks deferral information.
+    # Check all tasks in the deferral queue and add them back to the waiting
+    # list if they are no longer deferred.
+    sub _process_deferrals {
+        my ($self) = @_;
+
+        # clean up the current deferral object for the tasks being processed.
+        $self->{defer_obj} = {};
+        foreach my $task ( @{ $self->{processing_list} } ) {
+            $self->{defer_obj}->{$_} = 1 foreach _get_task_processor( $task )->deferral_tags( $task );
+        }
+
+        # Separate deferred tasks from non-deferred tasks.
+        my @defer;
+        my @proc;
+        foreach my $task ( @{ $self->{deferral_queue} } ) {
+            if( _get_task_processor( $task )->is_task_deferred( $task, $self->{defer_obj} ) ) {
+                push @defer, $task;
+            }
+            else {
+                # move 'no longer deferred' tasks in reverse order to processing list
+                unshift @proc, $task;
+            }
+        }
+
+        # update queues
+        $self->{queue_waiting}  = [ @proc, @{ $self->{queue_waiting} } ] if @proc;
+        $self->{deferral_queue} = \@defer;
+        return;
+    }
+
+
     # Handle the case of too many tasks being processed
     # Are there too many in processing?
     #    check to see if any processes are complete, and remove them.
@@ -620,6 +713,7 @@ my $taskqueue_uuid = "TaskQueue";
             $self->{processing_list} = [
                 grep { $_->pid() != $pid } @{$self->{processing_list}}
             ];
+            $self->_process_deferrals();
             $guard->update_file();
         }
         $self->_clean_completed_tasks( $guard );
@@ -634,7 +728,7 @@ my $taskqueue_uuid = "TaskQueue";
 # One guaranteed processor, the no-operation case.
 __PACKAGE__->register_task_processor( 'noop', sub {} );
 
-1;    # Magic true value required at the end of the module
+1;
 
 __END__
 
